#include "OgreOcclusionCullingOctreeSceneManager.h"
#include "OgreMath.h"
#include "OgreIteratorWrappers.h"
#include "OgreRenderSystem.h"
#include "OgreCamera.h"

#include <windows.h>

namespace Ogre {
	//-----------------------------------------------------------------------
	OcclusionCullingOctreeSceneManager::OcclusionCullingOctreeSceneManager()
	{
		mOcclusionCullingOctreeSceneTraverser = 
			new OcclusionCullingOctreeSceneTraverser(this, mDestRenderSystem);

		//mDisplayNodes = true;
		//mShowBoundingBoxes = true;
		//mShowBoxes = true;
		mMaxDepth = 20;
	}
	//-----------------------------------------------------------------------
	OcclusionCullingOctreeSceneManager::~OcclusionCullingOctreeSceneManager()
	{
		delete mOcclusionCullingOctreeSceneTraverser;
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_renderVisibleObjects(void)
	{
		//-- render background
		clearSpecialCaseRenderQueues();
		addSpecialCaseRenderQueue(RENDER_QUEUE_BACKGROUND);
		addSpecialCaseRenderQueue(RENDER_QUEUE_SKIES_EARLY);
	
		setSpecialCaseRenderQueueMode(SceneManager::SCRQM_INCLUDE);
		SceneManager::_renderVisibleObjects( );
		_deleteRenderedQueueGroups();

		//-- render visible objects (i.e., all but overlay)
		clearSpecialCaseRenderQueues();
		addSpecialCaseRenderQueue(RENDER_QUEUE_OVERLAY);
		setSpecialCaseRenderQueueMode(SceneManager::SCRQM_EXCLUDE);

		mOcclusionCullingOctreeSceneTraverser->setSceneRoot(mOctree);
		mOcclusionCullingOctreeSceneTraverser->renderScene(mCameraInProgress);
		
		_deleteRenderedQueueGroups();

		//-- render overlay
		clearSpecialCaseRenderQueues();
		SceneManager::_renderVisibleObjects( );
	
		clearSpecialCaseRenderQueues();
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_findVisibleObjects(Camera* cam, bool onlyShadowCasters)
	{
		// must be empty because objects are found and rendered in an interleaved fashion
		// in _renderVisibibleObjects 
		//char *msg = "now finding visible objects\n"; OutputDebugString(msg);
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_updateSceneGraph(Camera* cam)
	{
		//char *msg = "now updating scenegraph\n"; OutputDebugString(msg);
		//mOcclusionCullingOctreeSceneTraverser->preprocess();

     	mOcclusionCullingOctreeSceneTraverser->setNumSceneNodes((int)mSceneNodes.size());
		mOcclusionCullingOctreeSceneTraverser->setRenderSystem(mDestRenderSystem);
        
		mOcclusionCullingOctreeSceneTraverser->setNumOctreeNodes(mNumOctreeNodes);

		OctreeSceneManager::_updateSceneGraph(cam);
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::setOption( const String & key, const void * val )
	{
		return mOcclusionCullingOctreeSceneTraverser->setOption( key, val) || OctreeSceneManager::setOption( key, val );
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOption( const String & key, void *val )
	{
		return mOcclusionCullingOctreeSceneTraverser->getOption( key, val) || OctreeSceneManager::getOption( key, val ) ;
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOptionValues( const String & key, StringVector  &refValueList )
	{
		return OctreeSceneManager::getOptionValues( key, refValueList );
	}
	//-----------------------------------------------------------------------
	bool OcclusionCullingOctreeSceneManager::getOptionKeys( StringVector & refKeys )
	{
		return mOcclusionCullingOctreeSceneTraverser->getOptionKeys ( refKeys ) || OctreeSceneManager::getOptionKeys( refKeys );
	}
	//-----------------------------------------------------------------------
	void OcclusionCullingOctreeSceneManager::_renderOctant(Camera *cam, Octree *octant)
	{
		//Add stuff to be rendered;
        NodeList::iterator it = octant->mNodes.begin();
         
        while(it != octant->mNodes.end())
        {
            OctreeNode *sn = *it;

            mNumObjects++;

			// check bounding box visibility of scene nodes
			if (cam->isVisible(sn->_getWorldAABB()))
			{
				sn->_addToRenderQueue(cam, getRenderQueue(), false);
				//node->_findVisibleObjects(cam, getRenderQueue(), false, mDisplayNodes, false);
				mVisible.push_back(sn);

				if (mDisplayNodes)
					getRenderQueue()->addRenderable(sn);

				// check if the scene manager or this node wants the bounding box shown.
				if (sn->getShowBoundingBox() || mShowBoundingBoxes)
					sn->_addBoundingBoxToQueue(getRenderQueue());

				if (mShowBoxes)
					getRenderQueue()->addRenderable(octant->getWireBoundingBox());
			}
            ++it;
        }

		SceneManager::_renderVisibleObjects();
	
		// delete all rendered objects from renderqueue
		_deleteRenderedQueueGroups();
	}
}	
